

class Modifiers(object):

    def __init__(self):
        pass

    abstract = ':abstract'
    serialized = ':serialized'
    static = ':static'
    const = ':const'
    external = ':external'
    visitor = ':visitor'
    set_function = ':set_function'
    runtime = ':runtime'
    key = ':key'
    link = ':link'
    storage = ':storage'
    private = ':private'
    protected = ':protected'
    public = ':public'
    numeric = ':numeric'
    test = ':test'
    inline = ':inline'
    virtual = ':virtual'
    pointer = ':pointer'
    ref = ':ref'
    generate = ':generate'
    prefer_use_forward_declarations = ':prefer_use_forward_declarations'

    client = ':client'
    server = ':server'
    side_client = 'client'
    side_server = 'server'

